-----------------------------------------------------------------------------------------------
-- Client Lua Script for StatsPanel
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"

-----------------------------------------------------------------------------------------------
-- StatsPanel Module Definition
-----------------------------------------------------------------------------------------------
local StatsPanel = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function StatsPanel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.stats = {}

	o.stats.cash = 0;
	o.stats.looteditems = 0;
	o.stats.deaths = 0;
	o.stats.healingdone = 0;
	o.stats.damagedone = 0;
	o.stats.healingtaken = 0;
	o.stats.damagetaken = 0;
	
	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected

    return o
end

function StatsPanel:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterSlashCommand("stats", "OnStatsPanelOn", self)
	Apollo.RegisterEventHandler("LootedItem", "OnLootedItem", self)
	Apollo.RegisterEventHandler("LootedMoney", "OnLootedMoney", self)
	Apollo.RegisterEventHandler("DamageOrHealingDone", "OnDamageOrHealing", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("StatsPanelOn", "OnStatsPanelOn", self)
	Apollo.RegisterEventHandler("CombatLogDeath", "OnCombatLogDeath", self)

end
 

-----------------------------------------------------------------------------------------------
-- StatsPanel OnLoad
-----------------------------------------------------------------------------------------------
function StatsPanel:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("StatsPanel.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- StatsPanel OnDocLoaded
-----------------------------------------------------------------------------------------------
function StatsPanel:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "StatsPanelForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		-- item list
		self.wndItemList = self.wndMain:FindChild("ItemList")
	    self.wndMain:Show(false, true)
	end
end

function StatsPanel:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end 

    return self.stats
end

function StatsPanel:OnRestore(eLevel, tData)
	self.stats = tData
end
-----------------------------------------------------------------------------------------------
-- StatsPanel Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/stats"
function StatsPanel:OnStatsPanelOn()
	self.wndMain:Invoke()
	self:PopulateItemList()
end

function StatsPanel:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Statistics", {"StatsPanelOn", "", nil})
end

function StatsPanel:OnCombatLogDeath(deadUnit)
	if GameLib.IsControlledUnit(deadUnit.unitCaster) then
		self.stats.deaths = self.stats.deaths + 1
		self:PopulateItemList()
	end
end

function StatsPanel:OnLootedMoney(monLooted)
	local eCurrencyType = monLooted:GetMoneyType()
	if eCurrencyType ~= Money.CodeEnumCurrencyType.Credits then
		return
	end
	self.stats.cash = self.stats.cash + monLooted:GetAmount()
	self:PopulateItemList()
end

function StatsPanel:OnLootedItem(itemlooted)
	self.stats.looteditems = self.stats.looteditems + 1
	self:PopulateItemList();
end

function StatsPanel:OnDamageOrHealing(unitCaster, unitTarget, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical)
	if nDamage == nil then
		return
	end
	
	if GameLib.IsControlledUnit(unitTarget) then
		local bHeal = eDamageType == GameLib.CodeEnumDamageType.Heal or eDamageType == GameLib.CodeEnumDamageType.HealShields
		if bHeal then
			self.stats.healingtaken = self.stats.healingtaken + (nDamage)
		else
			self.stats.damagetaken = self.stats.damagetaken + (nDamage)
		end
		self:PopulateItemList()
	end
	
	if GameLib.IsControlledUnit(unitCaster) then
		local bHeal = eDamageType == GameLib.CodeEnumDamageType.Heal or eDamageType == GameLib.CodeEnumDamageType.HealShields
		if bHeal then
			self.stats.healingdone = self.stats.healingdone + (nDamage)
		else
			self.stats.damagedone = self.stats.damagedone + (nDamage)
		end
		
		self:PopulateItemList()
	end
end

-----------------------------------------------------------------------------------------------
-- StatsPanelForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function StatsPanel:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function StatsPanel:OnCancel()
	self.wndMain:Close() -- hide the window
end


-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
-- populate item list
function StatsPanel:PopulateItemList()
	-- make sure the item list is empty to start with
	self:DestroyItemList()
	self:AddItem("Copper Coins Looted: " .. self.stats.cash, "cash");
	self:AddItem("Items Looted: " .. self.stats.looteditems, "looteditem");
	self:AddItem("Damage Done: " .. self.stats.damagedone, "damagedone");
	self:AddItem("Healing Done: " .. self.stats.healingdone, "healdone");
	self:AddItem("Damage Taken: " .. self.stats.damagetaken, "damagetaken");
	self:AddItem("Healing Taken: " .. self.stats.healingtaken, "healtaken");
	self:AddItem("Time's Died: " .. self.stats.deaths, "deaths")
	self:AddItem("Want to see other stats tracked? Let me know. Twitter: @Fusspawn", "contact"	);
	self.wndItemList:ArrangeChildrenVert()
end

-- clear the item list
function StatsPanel:DestroyItemList()
	-- destroy all the wnd inside the list
	for idx,wnd in pairs(self.tItems) do
		wnd:Destroy()
	end

	-- clear the list item array
	self.tItems = {}
	self.wndSelectedListItem = nil
end

-- add an item into the item list
function StatsPanel:AddItem(i, id)
	-- load the window item for the list item
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	
	-- keep track of the window item created
	self.tItems[id] = wnd

	-- give it a piece of data to refer to 
	local wndItemText = wnd:FindChild("Text")
	if wndItemText then -- make sure the text wnd exist
		wndItemText:SetText(i) -- set the item wnd's text to "item i"
		wndItemText:SetTextColor(kcrNormalText)
	end
	wnd:SetData(i)
end

-- when a list item is selected
function StatsPanel:OnListItemSelected(wndHandler, wndControl)
    -- make sure the wndControl is valid
    if wndHandler ~= wndControl then
        return
    end
    
    -- change the old item's text color back to normal color
    local wndItemText
    if self.wndSelectedListItem ~= nil then
        wndItemText = self.wndSelectedListItem:FindChild("Text")
        wndItemText:SetTextColor(kcrNormalText)
    end
    
	-- wndControl is the item selected - change its color to selected
	self.wndSelectedListItem = wndControl
	wndItemText = self.wndSelectedListItem:FindChild("Text")
    wndItemText:SetTextColor(kcrSelectedText)

end


-----------------------------------------------------------------------------------------------
-- StatsPanel Instance
-----------------------------------------------------------------------------------------------
local StatsPanelInst = StatsPanel:new()
StatsPanelInst:Init()
